package main

import (
	"os"
	"sync"
	"time"
)

type writeResults struct {
	Duration  time.Duration
	bytesTime float64
	Error     error
}

type readResults struct {
	Duration  time.Duration
	bytesTime float64
	data      []byte
	Error     error
}

func write(data []byte, key []byte, file string) (time.Duration, float64, error) {
	start := time.Now()

	encryptedData, err := encrypt(data, key)
	checkError(err)

	err = os.WriteFile(file, encryptedData, 0644)
	checkError(err)

	duration := time.Since(start)
	bytesTime := float64(len(encryptedData))

	return duration, bytesTime, nil

}

func writeConcurrent(data []byte, key []byte, file string, wg *sync.WaitGroup, resultChannel chan<- writeResults) {

	defer wg.Done()

	start := time.Now()

	encryptedData, err := encrypt(data, key)
	checkError(err)

	err = os.WriteFile(file, encryptedData, 0644)
	checkError(err)

	duration := time.Since(start)
	bytesTime := float64(len(encryptedData))

	resultChannel <- writeResults{
		Duration:  duration,
		bytesTime: bytesTime,
		Error:     nil,
	}

}

func read(file string, key []byte) (time.Duration, float64, []byte, error) {

	start := time.Now()

	encryptedData, err := os.ReadFile(file)
	checkError(err)

	decryptData, err := decrypt(encryptedData, key)
	checkError(err)

	duration := time.Since(start)
	bytesTime := float64(len(encryptedData))

	return duration, bytesTime, decryptData, nil

}

func readConcurrent(file string, key []byte, wg *sync.WaitGroup, resultChannel chan<- readResults) {

	defer wg.Done()

	start := time.Now()

	encryptedData, err := os.ReadFile(file)
	checkError(err)

	decryptData, err := decrypt(encryptedData, key)
	checkError(err)

	duration := time.Since(start)
	bytesTime := float64(len(encryptedData))

	resultChannel <- readResults{
		Duration:  duration,
		bytesTime: bytesTime,
		data:      decryptData,
		Error:     nil,
	}

}

func latency(data []byte, key []byte, file string) (time.Duration, error) {
	start := time.Now()

	_, _, err := write(data, key, file)

	checkError(err)

	_, _, _, err = read(file, key)
	checkError(err)

	return time.Since(start), nil

}
